package Perlito5::JavaScript2::Lib;
use strict;

sub init {
    @Perlito5::Grammar::Use::Perlito_internal_lib_directory = (
        'Perlito5X/JavaScript',
        'Perlito5X',
        '',
    );
}

1;

